@@ -263,7 +263,8 @@ append_message_data (CamelFolder *folder, GByteArray *message,
 		stream_cache = camel_data_cache_add (exch->cache,
 						     "cache", new_uid, NULL);
 		if (stream_cache) {
-			camel_stream_write (stream_cache, message->data,
+			camel_stream_write (stream_cache,
+					    (char *) message->data,
 					    message->len);
 			camel_stream_flush (stream_cache);
 			camel_object_unref (CAMEL_OBJECT (stream_cache));
@@ -284,6 +285,33 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 	CamelStream *stream_mem;
 	CamelExchangeStore *store = CAMEL_EXCHANGE_STORE (folder->parent_store);
 
+	char *old_subject = NULL;
+	GString *new_subject;
+	int i, len;
+
+	/* 
+	   FIXME: We should add a top-level camel API camel_mime_message_prune_invalid_chars 
+	   which each of the provider will have to implement to remove things 
+	   that are invalid for their Transport mechanism. This will help in 
+	   avoiding  duplication of work. Now Sending and Importing both requires
+	   substitution of \t and \n with blank. 
+	*/
+	
+	old_subject = g_strdup(camel_mime_message_get_subject (message));
+	
+	if (old_subject) {
+		len = strlen (old_subject);
+		new_subject = g_string_new("");
+		for (i = 0; i < len; i++) 
+			if ((old_subject[i] != '\t') && (old_subject[i] != '\n'))
+				new_subject = g_string_append_c (new_subject, old_subject[i]);
+			else
+				new_subject = g_string_append_c (new_subject, ' ');
+		camel_mime_message_set_subject (message, new_subject->str);
+		g_free (old_subject);
+		g_string_free (new_subject, TRUE);
+	}	
+
 	if (!camel_exchange_store_connected (store, ex)) {
 		camel_exchange_journal_append ((CamelExchangeJournal *) ((CamelExchangeFolder *)folder)->journal, message, info, appended_uid, ex);
 		return;
@@ -395,7 +423,7 @@ get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 		return NULL;
 	}
 
-	camel_stream_write (stream, ba->data, ba->len);
+	camel_stream_write (stream, (char *) ba->data, ba->len);
 	camel_stream_flush (stream);
 	camel_object_unref (CAMEL_OBJECT (stream));
 
