@@ -512,7 +512,7 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 		gchar *key;
 		g_mutex_lock (exch->folders_lock);
 		if (g_hash_table_lookup_extended (exch->folders, folder_name,
-						  (gpointer *) &key, NULL)) {
+						  (gpointer) &key, NULL)) {
 			g_hash_table_remove (exch->folders, key);
 			g_free (key);
 		}
@@ -535,10 +535,10 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 static gboolean
 exchange_folder_subscribed (CamelStore *store, const char *folder_name)
 {
-	d(printf ("is subscribed folder : %s\n", folder_name));
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	guint32 is_subscribed;
 	
+	d(printf ("is subscribed folder : %s\n", folder_name));
 	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
 		return FALSE;
 	}
@@ -558,9 +558,9 @@ static void
 exchange_subscribe_folder (CamelStore *store, const char *folder_name,
 				CamelException *ex)
 {
-	d(printf ("subscribe folder : %s\n", folder_name));
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	
+	d(printf ("subscribe folder : %s\n", folder_name));
 	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot subscribe folder in offline mode."));
 		return;
@@ -575,9 +575,9 @@ static void
 exchange_unsubscribe_folder (CamelStore *store, const char *folder_name,
 				CamelException *ex)
 {
-	d(printf ("unsubscribe folder : %s\n", folder_name));
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	
+	d(printf ("unsubscribe folder : %s\n", folder_name));
 	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot unsubscribe folder in offline mode."));
 		return;
@@ -612,13 +612,13 @@ static CamelFolderInfo *
 make_folder_info (CamelExchangeStore *exch, char *name, char *uri,
 		  int unread_count, int flags)
 {
-	d(printf ("make folder info : %s flags : %d\n", name, flags));
 	CamelFolderInfo *info;
 	const char *path;
 	gchar **components;
 	char *new_uri;
 	char *temp;
 
+	d(printf ("make folder info : %s flags : %d\n", name, flags));
 	path = strstr (uri, "://");
 	if (!path)
 		return NULL;
@@ -665,6 +665,9 @@ make_folder_info (CamelExchangeStore *exch, char *name, char *uri,
 		info->flags |= CAMEL_FOLDER_SUBSCRIBED;
 		d(printf ("MMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMMmark as subscribed\n"));
 	}
+
+	if (flags & CAMEL_STUB_FOLDER_NOCHILDREN)
+		info->flags |= CAMEL_FOLDER_NOCHILDREN;
 	return info;
 }
 
@@ -681,7 +684,8 @@ postprocess_tree (CamelFolderInfo *info)
 	/* If the node still has children, keep it */
 	if (info->child)
 		return info;
-	info->flags |= CAMEL_FOLDER_NOCHILDREN;
+
+	/* info->flags |= CAMEL_FOLDER_NOCHILDREN; */
 
 	/* If it's a mail folder (not noselect), keep it */
 	if (!(info->flags & CAMEL_FOLDER_NOSELECT))
@@ -726,6 +730,14 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 	if (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED)
 		store_flags |= CAMEL_STUB_STORE_FOLDER_INFO_SUBSCRIBED;
 
+	/* CAMEL_STORE_FOLDER_INFO_FAST is called on only subscribed folders
+	   and the stub implementation checks for subscribed and recursive
+	   flags alone and hence, we need to OR SUBSCRIBED flag as well
+	 */
+	if (flags & CAMEL_STORE_FOLDER_INFO_FAST)
+		store_flags |= (CAMEL_STUB_STORE_FOLDER_INFO_SUBSCRIBED | 
+				CAMEL_STUB_STORE_FOLDER_INFO_FAST);
+
 	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_FOLDER_INFO,
 			      CAMEL_STUB_ARG_STRING, top,
 			      CAMEL_STUB_ARG_UINT32, store_flags,
